package giden.contrib.TSP;
import giden.CORE.*;
import giden.GRDS.*;
import java.awt.Color;
import java.lang.Math.*;

/*
 * WE MAKE NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. WE SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY USER AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * TspSolverBase is an abstract base class for implementing TSP solvers.
 * It provides these basic services:
 * <ul>
 * <li>Manages basic solver execution including the Order of the nodes through the tour and results tracking
 * <li>Creates edges between all nodes
 * <li>Creates 2D NodeArrays that, for each pair of nodes:
 * <ul>
 * <li> Retrieves the edge between the nodes
 * <li> Retrieves the distance between the nodes
 * </ul>
 * <li>Comptues the distance through the tour
 * <li>Draw the completed tour and displays the distance on the status line
 * </ul>
 * <p>
 * Copyright (c) 2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * @see TspSolverExhaustive
 * @see TspSolverNearestNeighbor
 *
 * @version 	1.0a 2004 Feb 01
 * @author 	    D. S. Dilworth
 */

public abstract class TspSolverBase extends ExecBase
//public class TspSolverBase extends ExecBase
{
    public UserBase ControllerUI;
    /** User interactions */
    public UserBase UI;
    /** Access to the network */
    public Network Net;
    /** Used to delete nodes or edges created by a solver */
    GraphState State;
    /** All nodes are initially put in this set */
    AnimationSet InitialSet;
    /** Annotation for final tour */
    FinalSet Tour;
    /** Returns the edge between two nodes */
    public NodeArray2dEdge NodesToEdge;
    /** Returns the distance between two nodes */
    public NodeArray2dInt NodesToDistance;
    /** The number of nodes in the network */
    public int NodeCount;
    /** The order of nodes in tour, the first node is at index 0 */
    public Node Order[];
    /** List of nodes, in tour order */
    LinkList TspNodes;
    /** Length of the minimum tour */
    int TspLength;
    /** True if the network is valid.  Valid means two or more nodes and no edges */
    boolean ValidNetwork;
   
    
    /** Initializes invariant objects */
    public TspSolverBase (UserBase userinfo)
    {
        super(userinfo);        // mandatory call to the base class
        UI=userinfo;            // local access for user interactions
        InitialSet = UI.CreateAnimationSet("",Color.lightGray); // initial set for nodes
        Tour = UI.CreateFinalSet("Tour",Color.green);           // final set for tour
        ControllerUI=null;
    }
    
    /** Prepares for solver execution */
    public boolean Setup(Network net)
    {
        Net=net;                        // local access to the network
        State = new GraphState(Net);    // get state so created edges can be removed
        
        NodeCount = Net.GetNodeCount(); // count of nodes in network
        if(NodeCount<2)                 // check two or more nodes
        {
            UI.UserMouseNodeInput("Setup: Solvers needs two or more nodes (click anywhere)");          
            ValidNetwork=false;
            return false;
        }
        if(Net.GetEdgeCount()!=0)       // check no edges
        {
            UI.UserMouseNodeInput("Setup: Network should have no edges (click anywhere)");          
            ValidNetwork=false;
            return false;
        }        
        Order = new Node[NodeCount];    // order of nodes in tour
        for(int i=0;i<NodeCount;i++)    // clear the order array
            Order[i]=null;
        EdgeArray distance_array = Net.GetEdgeArray("Euclidean Length");
        FillEdgeArray(distance_array);  // fill the 2D edge arrays
        UI.PutDirected(false);          // draw as undirected
        TspNodes = null;                // clear node tour list
        TspLength = Integer.MAX_VALUE;              // set length at maximum
        
        // prepare display for execution
        UI.MoveAll(InitialSet);                     // move nodes to initial set
        NodeListIter nit = Net.GetNodeListIter();   // iterator for nodes
        for(Node node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
            UI.PutLabel(node,"");                   // clear the node label    
        ValidNetwork=true;                          // set the valid network flag
        return true;
    }
        
    /** Executes the solver */
    public boolean Execute()
    {
        if(ValidNetwork==false)             // do we have a valid network?
            return false;
        UI.StatusLine("Executing");         // show the solver is executing
        System.out.println("In super.execute()\n") ;
        if ( ExecuteTspSolver()== false ) return false ;                 // execute the solver
        Evaluate();                         // evaluate tour
        DrawFinalTour();                    // draw the final tour
        UI.StatusLine("Tour = "+TspLength); // display length
        return true;       
    }
    
    /** Solves TSP for the given network and setup */
    public abstract boolean ExecuteTspSolver();
    /*
    boolean ExecuteTspSolver()
    {
        return true;
    }
    */

    /** Cleanup after solver execution is completed */
    public boolean Shutdown()
    {
        super.Shutdown();
        State.Restore();
        return true;
    }
    
    public void SaveResultsToNetwork(String name)
    {
        String results = new Integer(TspLength).toString();
        for(int i=0;i<NodeCount;i++)
            results=results+",N"+Order[i].GetIndex();
        Net.AddNamedObject(name,results);
    }

    /** Creates and fills the 2D Node arrays */
    void FillEdgeArray(EdgeArray distance_array)
    {
        NodesToEdge = new NodeArray2dEdge(Net);
        NodesToDistance = new NodeArray2dInt(Net);
        int dist;
        
        NodeListIter nit1 = Net.GetNodeListIter();    // outer loop iterator
        for(Node node1=nit1.GetFirstNode();node1!=null;node1=nit1.GetNextNode())
        {
            NodeListIter nit2 = Net.GetNodeListIter();    // inner loop iterator
            for(Node node2=nit2.GetFirstNode();node2!=null;node2=nit2.GetNextNode())
            {
                if(node1.GetIndex() < node2.GetIndex())     // check indices
                {
                    Edge edge = Net.CreateEdge(node1,node2);    // create the edge
                    
                    if(ControllerUI!=null)
                    {
                        UiNodeInfo info1 = ControllerUI.GetNodeInfo(node1);
                        UiNodeInfo info2 = ControllerUI.GetNodeInfo(node2);
                        int dx = info2.x - info1.x;
                        int dy = info2.y - info1.y;
                        dist = (int)Math.sqrt(dx*dx+dy*dy);
                    }
                    else
                    {
                        UI.AddEdge(edge);               // add edge to display                      
                        dist = distance_array.GetInt(edge);     // get distance for edge  
                    }
                    SingleStepVisible(false,edge,InitialSet);   // make it invisible, etc
                    UI.PutLabel(edge,"");                       // no lable on edge
                    NodesToEdge.PutEdge(node1,node2,edge);      // add edge to array(n1,n2)
                    NodesToEdge.PutEdge(node2,node1,edge);      // add edge to array(n2,n1)
                    NodesToDistance.PutInt(node1,node2,dist);   // add dist to array(n1,n2)
                    NodesToDistance.PutInt(node2,node1,dist);   // add dist to array(n2,n1)
                }
                else if(node1==node2)                       // trap n1==n2
                {
                    NodesToEdge.PutEdge(node1,node2,null);  // edge is null
                    NodesToDistance.PutInt(node1,node2,0);  // distance is zero
                }
            }
        }
    }
    
    /** Simulates animation sets with visibility */
    public void SingleStepVisible(boolean visible, Edge edge, AnimationSet set)
    {
        UI.PutEdgeVisibility(edge,visible);
        UI.SingleStep(edge,set);
    }
  
    /* Calculates the tour length and records the best tour */
    public int Evaluate()
    {
        int total=0;
        int length;
        Node node1,node2;
        LinkList nodelist = new SingleLinkList();
        
        for(int i=0;i<NodeCount;i++)
        {
            node1=Order[i];
            node2=Order[(i+1)%NodeCount];
            length = NodesToDistance.GetInt(node1,node2);
            total+=length;
            nodelist.AddItem(node1);
            /*
            int index1=node1.GetIndex();    // print each pair
            int index2=node2.GetIndex();
            System.out.println(" N"+index1+" N"+index2+" "+length); 
            */
        }
        if(total<TspLength)
        {
            TspLength=total;
            TspNodes = nodelist;
        }
        return total;
    }
    
    /** Draws the tour.  There is likely a simpler way to do this */
    void DrawFinalTour()
    {
        Edge edge;
        Node node1;
        Node node2 = null; 
        NodeListIter nit = new NodeListIter(TspNodes);
        for(node1=nit.GetFirstNode();node1!=null;node1=nit.GetNextNode())
        {
            UI.SingleStep(node1,Tour);
            if(node2!=null)
            {
                edge = NodesToEdge.GetEdge(node1,node2);
                SingleStepVisible(true,edge,Tour);
            }
            node2=node1;
        }
        node1 = nit.GetFirstNode();
        edge = NodesToEdge.GetEdge(node1,node2);
        SingleStepVisible(true,edge,Tour);  
    } 
    /** Returns the tour length */
    public int GetLength()
    {
        return TspLength;
    }
    public LinkList GetTour()
    {
        return TspNodes;
    }
};
