package giden.contrib.samples;
import giden.CORE.*;
import giden.GRDS.*;


import java.awt.Color;
import java.awt.Point;

/*
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 *
 * WE MAKE NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. WE SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY USER AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */


/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <p>
 * FeatureExample is an example program that shows how to
 * use many of the Network and Animation features of GIDEN.
 * The example has the following methods:
 * <li>FeatureExample, the constructor for this solver, called by GIDEN
 * <li>Setup, sets up the solver prior to each run, called by GIDEN
 * <li>Shutdown, terminates the solver, called by GIDEN
 * <li>Execute, runs the solver, called by GIDEN
 * <li>FeatureExampleSolver, executes the solver
 * <li>ClearColorsLabels, repaints the network
 * <li>StepNodes, steps through all nodes, highlighting the node and its edges.
 * <li>StepEdges, steps through all edges, highlighting the edge and its nodes.
 * <li>ClickNode, counts the number node clicks
 * <li>ClickEdge, counts the number edge clicks
 * <p>
 *
 * @see Network
 * @see SolverMenu
 * @see UserIO
 *
 * @version 	4.0a 2004 08 29
 * @author 	    D. S. Dilworth
 */


public class FeatureExample extends ExecBase
{
    Network Net;
    UiNetwork UiNet;
    UserBase UserInfo;    
    SolverServices Services;
           
    //--------------------
    // Animation variables
    //--------------------
    Color NodeSetColor;             // colors for trace sets
    Color EdgeSetColor;
    Color InitialColor;
    
    AnimationSet NodeSet;           // trace sets
    AnimationSet EdgeSet;
    AnimationSet InitialSet;
    
    NodeArray NodeClickCount;       // count of node clicks
    EdgeArray EdgeClickCount;       // count of edge clicks
    EdgeArray Length;               // edge length for PQ
       
    /** Initializes invariant objects */
    public FeatureExample(UserBase userinfo)
    {
        super(userinfo);
        UserInfo=userinfo;
        Services=UserInfo.Services;
        //--------------------------------------
        // Set the colors for the animation sets
        //--------------------------------------
        NodeSetColor=Color.red;
        EdgeSetColor=Color.green;
        InitialColor=Color.orange;
        InitialSet = UserInfo.CreateAnimationSet("",InitialColor);
        NodeSet    = UserInfo.CreateAnimationSet("ByNodes",NodeSetColor);
        EdgeSet    = UserInfo.CreateAnimationSet("ByEdges",EdgeSetColor);
    }
    
    /** Prepares for solver execution */
    public boolean Setup(Network net)
    {
        Net=net;
        Services.AddSolverMenuInfo();  // enable FeatureExample.html as the Help->Info file       
        UiNet = UserInfo.GetUiNet();
        NodeClickCount = new NodeArray(Net.GetCreatedNodesCount(),"NodeClickCount");
        NodeClickCount.Fill(0);
        EdgeClickCount = new EdgeArray(Net.GetCreatedEdgesCount(),"EdgeClickCount");
        EdgeClickCount.Fill(0);
                        
        ClearColorsLabels(false,false);
        Length = Net.GetEdgeArray("Euclidean Length");
        UserInfo.ShowPseudoCodeText("start");
        UserInfo.StatusLine("Setup FeatureExample");
        return true;
    }

    /** Discards solver resources */
    public boolean Shutdown()
    {
        NodeClickCount=null;
        EdgeClickCount=null;
        return true;
    }
    
    /** Executes the solver */
    public boolean Execute()
    {
        FeatureExampleSolver();
        UserInfo.CreateFinalButtons();
        UserInfo.StatusLine("Feature Example Ends");
        UserInfo.ShowPseudoCodeText("end");
        return true;
    }
    
    /** Steps through the four solver methods */ 
    public void FeatureExampleSolver()
    {
        StepNodes(Net);
        StepEdges(Net,Length);
        ClickNode();
        ClickEdge();
        ClearColorsLabels(true,true);
    }
    /** Clears the node/edge colors and labels.
     *  The NodeCount and EdgeCount labels are displayed if the
     *  corresponding boolean parameters are true */
    private void ClearColorsLabels(boolean nodecount, boolean edgecount)
    {
        Node node;
        Edge edge;
        NodeListIter nit;
        EdgeListIter eit;

        //-------------------------------------
        // clear labels for each node/edge
        //-------------------------------------
        UserInfo.MoveAll(InitialSet);   // node/edges -> InitialSet
        nit = Net.GetNodeListIter();    // node list iterator
        for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
        {
            if(nodecount)
                UserInfo.PutLabel(node,NodeClickCount.GetInt(node)); // use count as the label
            else
                UserInfo.PutLabel(node,""); // clear the node label
        }                
        eit = Net.GetEdgeListIter();    // edge list iterator
        for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())
        {
            if(edgecount)
                UserInfo.PutLabel(edge,EdgeClickCount.GetInt(edge)); // use count as the label
            else
                UserInfo.PutLabel(edge,""); // clear the edge label
        }            
    }
    /** Steps through all nodes, highlighting the node and its edges.
     * Nodes are processed in the order they are found in the network,
     * and are labeled with the number of edges in the node's edge list.
     * Edge are labeled in the order they are found in the node's edge list. */
    private void StepNodes(Network net)
    {
        int label;
        Node node;
        Edge edge;
        EdgeListIter eit;

        ClearColorsLabels(false,false);
        UserInfo.StatusLine("Processing Nodes by order in graph");
        UserInfo.ShowPseudoCodeText("stepnodes");
        
        NodeListIter nit = Net.GetNodeListIter();
        LinkList adjedges = new SingleLinkList();
        //-----------------------------------------------
        // color and label each node/edge, then clear
        // - each node and its adjacent edges are colored
        // - the node label shows the number of edges
        // - the edge label shows its index for the node
        //-----------------------------------------------
        for(node=nit.GetFirstNode();node!=null;node=nit.GetNextNode())
        {
            //---------------------------------
            // label the node and then color it
            //---------------------------------
            UserInfo.PutLabel(node,node.GetEdgeCount());
            UserInfo.SingleStep(node,NodeSet);
            //-----------------------------------------
            // color/label each edge, keep an edge list
            //-----------------------------------------
            label=1;
            adjedges.DeleteContents();          // empty the edge list
            eit = node.GetAdjListIter();    // iterator for adj edges
            for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())
            {
                adjedges.AddItem(edge);     // add edge to list
                UserInfo.PutLabel(edge,label++);
                UserInfo.SingleStep(edge,NodeSet);
            }
            UserInfo.Trace(NodeSet);        // trace the set
            UserInfo.Update();
            //-----------------------------------------
            // clear the color/label for each node/edge
            // - use the edge list for clearing
            //-----------------------------------------
            UserInfo.PutLabel(node,"");         // clear node label
            UserInfo.SingleStep(node,InitialSet);
            eit = new EdgeListIter(adjedges);   // edge list iterator
            for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())
            {                                   // clear edges
                UserInfo.PutLabel(edge,"");
                UserInfo.SingleStep(edge,InitialSet);
            }
        }        
    }
    
    /** Steps through all edges, highlighting the edge and its nodes.
     * A priority queue is used to sort edges by length.
     * Edges are labeled with their edge number, and the edge length.
     * Nodes are labeled with S or T, representing source or target, respectively */
    private void StepEdges(Network net, EdgeArray length)
    {
        PQList PQ = new PQList();       // empty priority queue
        EdgeListIter eit = Net.GetEdgeListIter();
        Edge edge;
        Node source,target;
        String label;
        String edgeindex,edgelength;

        ClearColorsLabels(false,false);
        UserInfo.StatusLine("Processing edges in a PQ");
        UserInfo.ShowPseudoCodeText("stepedges");
        
        //------------------------------
        // put edges and lengths in a PQ
        //------------------------------
        for(edge=eit.GetFirstEdge();edge!=null;edge=eit.GetNextEdge())
            PQ.Insert(edge,length.GetInt(edge));
        while(PQ.NotEmpty())
        {
            //------------------------------------------------
            // get edge from PQ, label(index,length) and color
            //------------------------------------------------
            edge   = PQ.DeleteMinEdge();
            edgeindex = String.valueOf(edge.GetIndex());
            edgelength = String.valueOf(length.GetInt(edge));
            label = "("+edgeindex+","+edgelength+")";
            UserInfo.PutLabel(edge,label);
            UserInfo.SingleStep(edge,EdgeSet);
            //--------------------------------------
            // mark the source node with label/color
            //--------------------------------------
            source = edge.GetSource();
            UserInfo.PutLabel(source,"S");
            UserInfo.SingleStep(source,EdgeSet);
            //--------------------------------------
            // mark the target node with label/color
            //--------------------------------------
            target = edge.GetTarget();
            UserInfo.PutLabel(target,"T");
            UserInfo.SingleStep(target,EdgeSet);
            UserInfo.Trace(EdgeSet);        // trace the set
            UserInfo.Update();
            //-----------------------------------------
            // clear the color/label for each edge/node
            //-----------------------------------------
            UserInfo.PutLabel(edge,"");     // edge
            UserInfo.SingleStep(edge,InitialSet);
            UserInfo.PutLabel(source,"");   // source node
            UserInfo.SingleStep(source,InitialSet);
            UserInfo.PutLabel(target,"");   // target node
            UserInfo.SingleStep(target,InitialSet);
        }
    }
    
    /** Counts and displays the number of mouse clicks on a node */
    private void ClickNode()
    {
        Node node;
        Edge edge;
        boolean loop;
        int count;

        ClearColorsLabels(true,false);
        UserInfo.StatusLine("Count node clicks");
        UserInfo.ShowPseudoCodeText("clicknode");
        //-----------------------------------------
        // loop on nodes until they click elsewhere
        //-----------------------------------------
        loop=true;
        while(loop)
        {
            // Note: mouse input automatically calls Update()
            node = UserInfo.UserMouseNodeInput("Click on a node, or elsewhere to end");
            if(node==null)
                loop=false;         // quit the loop
            else
            {
                // count clicks and display as a label
                count = NodeClickCount.GetInt(node);
                NodeClickCount.Put(node,++count);
                UserInfo.PutLabel(node,count);
                UserInfo.SingleStep(node,NodeSet);
            }
        }
    }
    
    /** Counts and displays the number of mouse clicks on an edge */
    private void ClickEdge()
    {
        Edge edge;
        boolean loop;
        int count;
        
        ClearColorsLabels(false,true);
        UserInfo.StatusLine("Count edge clicks");
        UserInfo.ShowPseudoCodeText("clickedge");
        //-----------------------------------------------------------------
        // loop on edges until the user clicks on something besides an edge
        //-----------------------------------------------------------------
         loop=true;
        while(loop)
        {
            // Note: mouse input automatically calls Update()
            edge = UserInfo.UserMouseEdgeInput("Click on an edge, or elsewhere to end");
            if(edge==null)
                loop=false;         // quit the loop
            else
            {
                // count clicks and display as a label
                count = EdgeClickCount.GetInt(edge);
                EdgeClickCount.Put(edge,++count);
                UserInfo.PutLabel(edge,count);
                UserInfo.SingleStep(edge,EdgeSet);
            }
        }
    }
    
}
    
    
