package giden.gidenSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Color;

/**********************************************************************/
// Capacity Scaling class
/**********************************************************************/
public class CapacityScaling extends MinCostFlow {

    private AnimationSet Path;
    private ColorSet     PreprocessEdge;

    private boolean feasible;

    private final int INFINITY = Integer.MAX_VALUE;
    private final int PERM_LABELED = 1;
    private final int NOT_PERM_LABELED = 2;

    /****************************************************************/
    // CapacityScaling constructor
    /****************************************************************/
    public CapacityScaling(UserBase userinfo) {
        super(userinfo);

        Path           = UI.CreateAnimationSet("Path",Color.blue);
        PreprocessEdge = UI.CreateColorSet(Color.red);
    }

    public String getSolverName() {return "Capacity Scaling";}

    /****************************************************************/
    // Setup
    //   - calls MinCostFlow base class Setup()
    //   - initializes solver specific information
    /****************************************************************/
    public boolean Setup(Network net) {
        if(super.Setup(net)==false)
            return false;

        InitializeNetworkLabels(true);
        feasible = true;

        return true;
    }

    /****************************************************************/
    // CapacityScaling Execute
    /****************************************************************/
    public boolean Execute() {

        //        EdgeArray Cost = GetCostArray();
        //        EdgeArray Capacity = GetCapacityArray();
        //        NodeArray Supply = GetSupplyArray();
        EdgeArray Flow = GetFlowArray();
        NodeArray Potential = GetPotentialArray();

        Network Net = GetNetwork();

        NodeArray Excess = new NodeArray(Net.GetNodeIndex(),"Excess");
        EdgeArray RedCost = new EdgeArray(Net.GetEdgeIndex(),"RedCost");

        CapacityScalingSolver(Net,Cost,Flow,Capacity,Potential,Supply,
                            Excess,RedCost);
        if (feasible) {
            DisplayFinalSolution(Net,Cost,Flow,Capacity,Potential,Supply);
            UI.Trace(Path);
            UI.Update();
        }

        return true;
    }

    /****************************************************************/
    /****************************************************************/
    private int CalculateDelta(Network   Net,
                               EdgeArray Capacity) {
        int U = 0;
        EdgeListIter eit = Net.GetEdgeListIter();

        // find the max capacity ("U" in AMO)
        for (Edge e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge())
            U = Math.max(U,Capacity.GetInt(e));

        // return value == 2^{floor(log_{2}(max(capacity[e])))}
        return (int) Math.pow(2.0,Math.floor(Math.log(U)/Math.log(2)));
    }

    /****************************************************************/
    // CapacityScaling CapacityScalingSolver
    /****************************************************************/
    private void CapacityScalingSolver(Network   Net,
                                       EdgeArray Cost,
                                       EdgeArray Flow,
                                       EdgeArray Capacity,
                                       NodeArray Potential,
                                       NodeArray Supply,
                                       NodeArray Excess,
                                       EdgeArray RedCost) {
        UI.StatusLine("Capacity Scaling Solver begins...");
        UI.ShowPseudoCodeText("start");
        UI.StatusLine("");


        Edge e;
        EdgeListIter eit;
        Node i;
        NodeListIter nit;


        Node s,t;
        BoundedQueue E = new BoundedQueue(Net.GetNodeIndex());
        NodeArray Dist = new NodeArray(Net.GetNodeIndex(),"Dist");
        NodeArray Pred = new NodeArray(Net.GetNodeIndex(),"Pred");
        NodeArray NodeStatus = new NodeArray(Net.GetNodeIndex(),"NodeStatus");
        

        Initialization(Net,Flow,Potential,Excess,Supply,Cost,RedCost);

        int DELTA = CalculateDelta(Net,Capacity);
        UI.StatusLine("Initial delta = "+DELTA);
        UI.ShowPseudoCodeText("calc delta");
        UI.StatusLine("");

        UI.ShowPseudoCodeText("delta loop");
        while (DELTA >= 1) { // DELTA-scaling phase
            UI.StatusLine("Current delta = "+DELTA);

            DeltaPhasePreprocessing(Net,DELTA,Flow,Capacity,
                                    Excess,Supply,Potential,E,Cost,RedCost);

            UI.ShowPseudoCodeText("initialize E");
            UI.ShowPseudoCodeText("initialize D");
            UI.ShowPseudoCodeText("augment loop");
            while (!E.IsEmpty()) {
                s = (Node) E.GetTop();
                UI.SingleStep(s,Path);
                UI.ShowPseudoCodeText("select node");

                t = FindDeltaAugmentingPath(Net,s,DELTA,Dist,Excess,
                                            Pred,NodeStatus,
                                            Flow,Capacity,RedCost);
                UI.ShowPseudoCodeText("path check");

                // check to be sure that a path was found
                if (t != null) {
                    UI.ShowPseudoCodeText("distance labels");

                    ShowPath(t,Pred);
                    UpdatePotentialsAndReducedCosts(Net,t,Dist,Excess,
                                                    Potential,NodeStatus,
                                                    Cost,Flow,Capacity,
                                                    RedCost);
                    AugmentFlow(s,t,DELTA,Excess,Pred,Potential,Cost,
                                RedCost,Flow,Capacity);

                    if (Excess.GetInt(s) < DELTA)
                        E.Pop();

                    UI.ShowPseudoCodeText("update all");
                }
                else {
                    E.Pop(); // no delta-augmenting path from s

                    if (DELTA == 1) { // infeasible problem
                        UI.StatusLine("Infeasible problem --- "
                                      +"no augmenting path with delta = 1.");
                        UI.ShowPseudoCodeText("remove k");
                        UI.StatusLine("");
                        UI.SingleStep(s,InitialSet);

                        UI.ShowPseudoCodeText("end");

                        ReportInfeasibility(Net,NodeStatus,Supply,Capacity);
                        return;
                    }
                    UI.ShowPseudoCodeText("remove k");
                    UI.SingleStep(s,InitialSet);
                }
                UI.ShowPseudoCodeText("augment loop");
            }

            DELTA /= 2;

            // since using integer arith, we need to lie about DELTA == 0
            UI.StatusLine("New delta = "+((DELTA == 0)?"(1/2)":""+DELTA));

            UI.ShowPseudoCodeText("reduce delta");
            UI.ShowPseudoCodeText("delta loop");
        }
        UI.ShowPseudoCodeText("end");
    }


    /****************************************************************/
    // update reduced costs on edges
    /****************************************************************/
    private void UpdatePotentialsAndReducedCosts(Network   Net,
                                                 Node      t,
                                                 NodeArray Dist,
                                                 NodeArray Excess,
                                                 NodeArray Potential,
                                                 NodeArray NodeStatus,
                                                 EdgeArray Cost,
                                                 EdgeArray Flow,
                                                 EdgeArray Capacity,
                                                 EdgeArray RedCost) {
        NodeListIter nit = Net.GetNodeListIter();
        for (Node i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode()) {
            if (NodeStatus.GetInt(i) == PERM_LABELED)
                Potential.Put(i,Potential.GetInt(i)+Dist.GetInt(t)-Dist.GetInt(i));
            PutNodeLabel(i,Excess,Potential);
        }
        UI.ShowPseudoCodeText("update pi");

        EdgeListIter eit = Net.GetEdgeListIter();
        for (Edge e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
            RedCost.Put(e,Cost.GetInt(e)
                        -Potential.GetInt(e.GetSource())
                        +Potential.GetInt(e.GetTarget()));
            //            PutEdgeLabel(e,Flow,Capacity,RedCost);
            PutEdgeLabel(e,Flow,Capacity,Cost);
        }
    }

    /****************************************************************/
    // Graphically display the path between excess and deficit nodes
    /****************************************************************/
    private Node ShowPath(Node t, NodeArray Pred) {
        Edge e;
        Node i = t;
        while (Pred.GetEdge(i) != null) {
            UI.SingleStep(i,Path);
            e = Pred.GetEdge(i);
            UI.SingleStep(e,Path);
            i = e.GetOpposite(i);
        }
        UI.SingleStep(i,Path);
        UI.Trace(Path);
        UI.ShowPseudoCodeText("shortest path");
        UI.Update();
        return i;
    }

    /****************************************************************/
    // AugmentFlow -- send flow along s-t path
    /****************************************************************/
    private void AugmentFlow(Node      s,
                             Node      t,
                             int       DELTA,
                             NodeArray Excess,
                             NodeArray Pred,
                             NodeArray Potential,
                             EdgeArray Cost,
                             EdgeArray RedCost,
                             EdgeArray Flow,
                             EdgeArray Capacity) {
        Edge e;
        Node i;
        int delta;

        // augment by DELTA units
        delta = DELTA;
        // augment by as much as possible
        /*
        delta = Math.min(Excess.GetInt(s),-Excess.GetInt(t));
        i = t;
        while (i != s) {
            e = Pred.GetEdge(i);
            
            if (e.GetSource() == i && Flow.GetInt(e) < delta)
                delta = Flow.GetInt(e);
            else if (e.GetTarget() == i && Capacity.GetInt(e)-Flow.GetInt(e) < delta)
                delta = Capacity.GetInt(e)-Flow.GetInt(e);
            
            i = e.GetOpposite(i);
        }
        */

        i = t;
        while (Pred.GetEdge(i) != null) {
            e = Pred.GetEdge(i);

            if (e.GetSource() == i) {
                FlowCost -= delta*Cost.GetInt(e);
                Flow.Put(e, Flow.GetInt(e)-delta);
            }
            else {
                FlowCost += delta*Cost.GetInt(e);
                Flow.Put(e, Flow.GetInt(e)+delta);
            }

            i = e.GetOpposite(i);
        }

        Excess.Put(s,Excess.GetInt(s)-delta);
        Excess.Put(t,Excess.GetInt(t)+delta);

        UI.MoveAll(InitialSet);
        PutNodeLabel(s,Excess,Potential);
        PutNodeLabel(t,Excess,Potential);
        i = t;
        while (Pred.GetEdge(i) != null) {
            e = Pred.GetEdge(i);
            //            PutEdgeLabel(e,Flow,Capacity,RedCost);
            PutEdgeLabel(e,Flow,Capacity,Cost);
            i = e.GetOpposite(i);
        }
        FlowValue += delta;
        UI.Trace(Path);
        UI.ShowPseudoCodeText("augment");
        UI.Update();

    }
    

    /****************************************************************/
    // FindDeltaAugmentingPath -- finds aug path & returns deficit node
    /****************************************************************/
    private Node FindDeltaAugmentingPath(Network   Net,
                                         Node      s,
                                         int       DELTA,
                                         NodeArray Dist,
                                         NodeArray Excess,
                                         NodeArray Pred,
                                         NodeArray NodeStatus,
                                         EdgeArray Flow,
                                         EdgeArray Capacity,
                                         EdgeArray RedCost) {
        PQList PQ = new PQList();
        int temp_cost;
        Node i,j;
        Edge e;
        EdgeListIter eit;

        NodeStatus.Fill(NOT_PERM_LABELED);
        Pred.Fill(null);
        Dist.Fill(INFINITY);
        Dist.Put(s,0);

        NodeListIter nit = Net.GetNodeListIter();
        for (i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode()) 
            PQ.Insert(i,Dist.GetInt(i));

        while (!PQ.IsEmpty()) {
            i = PQ.DeleteMinNode();

            if (Dist.GetInt(i) == INFINITY) // infeasible problem
                continue;

            NodeStatus.Put(i,PERM_LABELED);

            if (Excess.GetInt(i) <= -DELTA && Dist.GetInt(i) != INFINITY)
                return i;

            eit = i.GetAdjListIter();
            for (e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
                temp_cost = INFINITY;

                if (e.GetSource() == i && Capacity.GetInt(e)-Flow.GetInt(e) >= DELTA)
                    temp_cost = Dist.GetInt(i) + RedCost.GetInt(e);
                else if (e.GetTarget() == i && Flow.GetInt(e) >= DELTA)
                    temp_cost = Dist.GetInt(i) - RedCost.GetInt(e);

                j = e.GetOpposite(i);
                if (temp_cost < Dist.GetInt(j)) {
                    Dist.Put(j,temp_cost);
                    Pred.Put(j,e);
                    PQ.DecreaseValue(j,temp_cost);
                }
            }
        }

        // we only exit here if the problem is infeasible;
        return (Node) null;
    }

    /****************************************************************/
    // ReportInfeasibility
    /****************************************************************/
    private void ReportInfeasibility(Network   Net,
                                     NodeArray NodeStatus,
                                     NodeArray Supply,
                                     EdgeArray Capacity) {
        //        SupplyNodes.PutLabel("");
        //        DemandNodes.PutLabel("");
        EdgesAtZero.PutLabel("");
        EdgesWithResidual.PutLabel("");
        EdgesAtCapacity.PutLabel("");

        ReachableNodes.PutLabel("Excess Reachable Nodes");
        InfeasibleCut.PutLabel("Infeasibility Cut");

        feasible = false;

        Node i;
        Edge e;
        int NetSupply = 0;  // net amount of supply for reachable node set
        int CutCapacity = 0;

        UI.MoveAll(InitialSet);

        boolean cut_visible = false;

        NodeListIter nit = Net.GetNodeListIter();
        for (i=nit.GetFirstNode() ; i != null ; i = nit.GetNextNode()) {
            PutNodeLabel(i,Supply);
            if (NodeStatus.GetInt(i) == PERM_LABELED) {
                UI.SingleStep(i,ReachableNodes);
                NetSupply += Supply.GetInt(i);

                EdgeListIter eit = i.GetOutListIter();
                for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge())
                    if (NodeStatus.GetInt(e.GetTarget()) != PERM_LABELED) {
                        UI.SingleStep(e,InfeasibleCut);
                        CutCapacity += Capacity.GetInt(e);
                        cut_visible = true;
                    }

                eit = i.GetInListIter();
                for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge())
                    if (NodeStatus.GetInt(e.GetSource()) != PERM_LABELED) {
                        UI.SingleStep(e,InfeasibleCut);
                        cut_visible = true;
                    }
            }
        }

        if (!cut_visible) {
            InfeasibleCut.PutLabel("");
            UI.StatusLine("Problem infeasible: "
                          +"Net supply of highlighted nodes = "
                          +NetSupply+" > 0 = (empty) cut capacity.");
        }
        else
            UI.StatusLine("Problem infeasible: "
                          +"Net supply of highlighted nodes = "
                          +NetSupply+" > "+CutCapacity+
                          " = cut capacity.");

        UI.Trace(InfeasibleCut);
        UI.Trace(ReachableNodes);
    }

    /****************************************************************/
    // DELTA-Preprocess
    /****************************************************************/
    private void DeltaPhasePreprocessing(Network      Net,
                                         int          DELTA,
                                         EdgeArray    Flow,
                                         EdgeArray    Capacity,
                                         NodeArray    Excess,
                                         NodeArray    Supply,
                                         NodeArray    Potential,
                                         BoundedQueue E,
                                         EdgeArray    Cost,
                                         EdgeArray    RedCost) {
        int residual;
        Edge e;
        EdgeListIter eit = Net.GetEdgeListIter();

        for (e=eit.GetFirstEdge() ; e!=null ; e=eit.GetNextEdge()) {

            UI.ShowPseudoCodeText("preprocess loop");
            UI.ShowPseudoCodeText("pp-check");
            
            if (Capacity.GetInt(e)-Flow.GetInt(e) >= DELTA 
                && RedCost.GetInt(e) < 0) {
                UI.SingleStep(e,PreprocessEdge);

                // send r_{ij} units along arc (i,j) to node j
                residual = Capacity.GetInt(e)-Flow.GetInt(e);
                FlowCost += residual*Cost.GetInt(e);
                FlowValue += residual;
                Flow.Put(e,Capacity.GetInt(e));
                Excess.Put(e.GetTarget(),Excess.GetInt(e.GetTarget())+residual);
                Excess.Put(e.GetSource(),Excess.GetInt(e.GetSource())-residual);
                PutEdgeLabel(e,Flow,Capacity,Cost);
                PutNodeLabel(e.GetSource(),Excess,Potential);
                PutNodeLabel(e.GetTarget(),Excess,Potential);
                UI.ShowPseudoCodeText("pp-augment");

                UI.SingleStep(e,InitialSet);
            }
            else if (Flow.GetInt(e) >= DELTA && RedCost.GetInt(e) > 0) {
                UI.SingleStep(e,PreprocessEdge);

                // send r_{ji} units along arc (j,i) in G(x) to node i
                residual = Flow.GetInt(e);
                FlowCost -= residual*Cost.GetInt(e);
                FlowValue -= residual;
                Flow.Put(e,0);
                Excess.Put(e.GetTarget(),Excess.GetInt(e.GetTarget())-residual);
                Excess.Put(e.GetSource(),Excess.GetInt(e.GetSource())+residual);
                PutEdgeLabel(e,Flow,Capacity,Cost);
                PutNodeLabel(e.GetSource(),Excess,Potential);
                PutNodeLabel(e.GetTarget(),Excess,Potential);
                UI.ShowPseudoCodeText("pp-augment");

                UI.SingleStep(e,InitialSet);
            }
        }

        Node i;
        NodeListIter nit = Net.GetNodeListIter();
        for (i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode()) {
            if (Excess.GetInt(i) >= DELTA) {
                    E.Append(i);
                    //UI.SingleStep(i,Path); // temp kludge for visual aid
            }
            else if (Excess.GetInt(i) <= -DELTA) {
                // DeficitNodes.Append(i);
            }
            else {
                // color node appropriately?
            }
        }

        // temp kludge for visual aid
        //for (i=nit.GetFirstNode() ; i!=null ; i=nit.GetNextNode()) 
        //    UI.SingleStep(i,InitialSet);
    }

    /****************************************************************/
    // Initialization
    /****************************************************************/
    private void Initialization(Network      Net,
                                EdgeArray    Flow,
                                NodeArray    Potential,
                                NodeArray    Excess,
                                NodeArray    Supply,
                                EdgeArray    Cost,
                                EdgeArray    RedCost) {
        Flow.Fill(0);
        Potential.Fill(0);

        NodeListIter nit = Net.GetNodeListIter();
        for (Node i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode())
            Excess.Put(i,Supply.GetInt(i));

        EdgeListIter eit = Net.GetEdgeListIter();
        for (Edge e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge())
            RedCost.Put(e,Cost.GetInt(e)); // since Potential.Fill(0);
        UI.ShowPseudoCodeText("initialize");
    }
}
