package  giden.gidenSOLVERS;
import giden.CORE.*;
import giden.gidenSOLVERS.*;
import giden.GRDS.*;
import giden.GUI.*;
//import giden.userSOLVERS.*;

import java.awt.Color;


/**********************************************************************/
// Shortest Path Problem base class
/**********************************************************************/
abstract class ShortestPath extends ExecBase {

    protected GidenSolverInput input;

    protected UserBase UI;

    // problem data
    protected Node Source;
    protected EdgeArray Length = null;
    protected NodeArray Pred;
    protected NodeArray Distance;
    protected int TreeLength;
    protected int NegCycleCost;


    private Network Net;

    private ColorSet InitialSet;
    private FinalSet SPTree;
    private FinalSet NegCycle;

    protected final int INFINITY = Integer.MAX_VALUE;


    /********************************************************************/
    // ShortestPath constructor
    /********************************************************************/
    ShortestPath(UserBase userinfo) {
        super(userinfo);
        UI=userinfo;
        InitialSet = UI.CreateColorSet(Color.lightGray);
        SPTree     = UI.CreateFinalSet("Shortest Path Tree", Color.orange);
        NegCycle   = UI.CreateFinalSet("Negative-Cost Cycle", Color.blue);

        input = new GidenSolverInput();
        input.addItem(SolverInputItem.INTEGER_EDGE_ARRAY,"Length","Weight");
    }

    abstract public String getSolverName();

    /********************************************************************/
    // Setup
    /********************************************************************/
    public boolean Setup(Network net) {
        if(net==null)
            return false;
        super.Setup(net);

        Net = net;
        UI.PutDirected(true);

        if (!UI.NullBase) {
            UI.CreateSolverInputDialog(getSolverName(), this, net, input);
            if (!input.isValidated())
                return false;

            // extract input info
            Object data = input.getItemData("Length");
            if (data instanceof EdgeArray)
                Length = (EdgeArray) data;
        }

        if (Length == null)
            Length = Net.GetEdgeArray("Weight");
        if (Length == null)
            Length = Net.GetEdgeArray("Euclidean Length");

        Pred = new NodeArray(Net.GetNodeIndex(),"Pred");
        Distance = new NodeArray(Net.GetNodeIndex(),"Distance");

        Distance.Fill(INFINITY);
        Pred.Fill(null);

        TreeLength = 0;
        NegCycleCost = 0;

        InitializeEdgeLabels();
        InitializeNodeLabels();

        UI.MoveAll(InitialSet);

        return true;
    }

    /********************************************************************/
    // Shutdown
    /********************************************************************/
    public boolean Shutdown() {
        Source = null;
        TreeLength = 0;
        NegCycleCost = 0;

        return super.Shutdown();
    }


    /********************************************************************/
    /********************************************************************/
    protected Node GetSourceNode() {
        if (Source == null)
            GetSource();

        return Source;
    }
    protected EdgeArray GetLengthArray() {return Length;}
    protected NodeArray GetDistanceArray() {return Distance;}
    protected NodeArray GetPredArray() {return Pred;}
    protected Network GetNetwork() {return Net;}

    /********************************************************************/
    private void GetSource() {
        Source = null;
        while (Source == null) {
            Source = UI.UserMouseNodeInput("Click on Source Node");
        }
        GIDEN.out.println("Source Node = "+Source.GetIndex());
    }

    /********************************************************************/
    protected void PutEdgeLabel(Edge e) {
        String label = "" + Length.GetInt(e);
        UI.PutLabel(e,label);
    }

    /********************************************************************/
    protected void PutNodeLabel(Node i) {
        if (i == Source)
            UI.PutLabel(i,"s");
        else if (Distance.GetInt(i) == INFINITY)
            UI.PutLabel(i,"-");
        else {
            String label = "" + Distance.GetInt(i);
            UI.PutLabel(i,label);
        }
    }

    /********************************************************************/
    private void InitializeNodeLabels() {
        // initialize node labels
        NodeListIter nit = Net.GetNodeListIter();
        for (Node i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode())
            PutNodeLabel(i);
    }

    /********************************************************************/
    private void InitializeEdgeLabels() {
        // initialize edge labels
        EdgeListIter eit = Net.GetEdgeListIter();
        for (Edge e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge())
            PutEdgeLabel(e);
    }

    
    /********************************************************************/
    protected boolean NegLengthValuesExist(Network Net, EdgeArray Length) {
        Edge e;

        EdgeListIter eit = Net.GetEdgeListIter();
        for (e = eit.GetFirstEdge() ; e != null ; e = eit.GetNextEdge()) {
            if (Length.GetInt(e) < 0)
                return true;
        }
        return false;
    }

    /********************************************************************/
    protected void DisplaySPTree(Network   Net,
                                 NodeArray Distance,
                                 NodeArray Pred,
                                 EdgeArray Length) {
        Node i;
        Edge e;
        int TmpTreeLength = 0;
        int SumShortestPaths = 0;

        UI.MoveAll(InitialSet);

        NodeListIter nit = Net.GetNodeListIter();
        for (i = nit.GetFirstNode() ; i != null ; i = nit.GetNextNode()) {
            if (Distance.GetInt(i) != INFINITY) {
                SumShortestPaths += Distance.GetInt(i);
                UI.SingleStep(i,SPTree);
                if (Pred.GetEdge(i) != null) {
                    UI.SingleStep(Pred.GetEdge(i),SPTree);
                    TmpTreeLength += Length.GetInt(Pred.GetEdge(i));
                }
            }
        }
        
        if (TmpTreeLength == TreeLength)
            UI.StatusLine("Shortest path tree length = "+TreeLength+
                                "; Sum of (finite) shortest path lengths = "+
                                SumShortestPaths);
        else
            UI.StatusLine("Error detected in tree length calculation!");

        UI.Trace(SPTree);
        UI.Update();

        NegCycle.PutEnabled(false);
        SPTree.PutEnabled(true);
    }

    /********************************************************************/
    protected void ExamineNegCycle(Network   Net,
                                   Node      i,
                                   NodeArray Pred,
                                   EdgeArray Length) {
        int CycleCost = 0;
        NodeArray Seen = new NodeArray(Net.GetCreatedNodesCount(),"Seen");
        Seen.Fill(Boolean.FALSE);


        Seen.Put(i,Boolean.TRUE);

        UI.MoveAll(InitialSet);

        // identify the cycle
        i = (Pred.GetEdge(i)).GetSource();
        while (Seen.GetObj(i).equals(Boolean.FALSE)) {
            Seen.Put(i,Boolean.TRUE);
            i = (Pred.GetEdge(i)).GetSource();
        }

        // now we know that i is in the cycle -- let's compute the cost
        Node j = i;
        Edge e = Pred.GetEdge(i);
        CycleCost = Length.GetInt(Pred.GetEdge(i));

        UI.SingleStep(i,NegCycle);
        UI.SingleStep(e,NegCycle);

        i = (Pred.GetEdge(i)).GetSource();
        UI.SingleStep(i,NegCycle);
        while (i != j) {
            e = Pred.GetEdge(i);
            UI.SingleStep(e,NegCycle);
            CycleCost += Length.GetInt(e);

            i = e.GetSource();
            UI.SingleStep(i,NegCycle);
        }

        GIDEN.out.println("Negative cost cycle detected:  Cycle cost = "
                           + CycleCost);
        UI.StatusLine("Negative cost cycle detected!  Cycle cost = "
                            +CycleCost);
        UI.Trace(NegCycle);
        UI.Update();

        NegCycle.PutEnabled(true);
        SPTree.PutEnabled(false);
    }

}
