package  giden.CORE;
import giden.gidenSOLVERS.*;
import giden.contrib.samples.*;
import giden.GRDS.*;
import giden.GUI.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Copyright (c) 1993-2004 C. R. Coullard, D. S. Dilworth, J. H. Owen,
 * All Rights Reserved.
 * <BR>
 * We make no representations or warranties about the suitability of
 * the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a
 * particular purpose, or non-infringement. We shall not be liable for
 * any damages suffered by user as a result of using, modifying or
 * distributing this software or its derivatives.
 * <p>
 RowDialogMain is a row-oriented dialog box that supports multiple
 objects derived from RowDialogEntry.  This object displays each
 row, interacts with GIDEN, displays controls defined by
 the solver, and interacts with the user.  Solvers retrieve the
 results of the interactions from the derived objects.
 
 The creation of a dialog box begins by creating a RowDialogMain object.
 Each row that is added to the dialog box inputs one type of information and returns
 an object that is used to retrieve the user selections.
 Currently these types rows can be added to the dialog box input:
 <ul>
 <li> A text string an be typed into a text field (AddRowText)
 <li> A filename can be typed into a text field or found by browsing (AddRowFilename)
 <li> An item can be selected from a pulldown menu of choices (AddRowChoice)
 <li> A node from a pulldown menu or by clicking on a node in the GIDEN window  (AddRowNode)
 <li> An edge from a pulldown menu or by clicking on an edge in the GIDEN window  (AddRowEdge)
 </ul>
 In addition to rows, the dialog box needs to have control buttons added.
 The buttons appear at the bottom of the dialog box and perform the [Cancel], [OK] functionality.

 After the input rows and control buttons are added, the Interact() method is called.
 The dialog box then takes control of the GIDEN environment,
 and it will return when the user has selected one of the control buttons.
 The text from the control button is returned from the Interact() method.
 The results from each row method can then be processed.
 
 The example below is taken from the DialogSolver.java solver.
 <pre>
    boolean InteractWithUser()
    {
        // create an empty dialog box
        RowDialogMain Dialog = UI.CreateRowDialog("Dialog Solver");
        
        // add the filename row 
        RowDialogText TextRow = Dialog.AddRowText("Test Text","default text");          
        
        // add the filename row 
        RowDialogFilename FilenameRow = Dialog.AddRowFilename("Test Filename","default.csv",false); 
        
        // add the choice row
        String Choices[] = new String[3];
        Choices[0]="Choice A";
        Choices[1]="Choice B";
        Choices[2]="Choice C";
        RowDialogChoice ChoiceRow = Dialog.AddRowChoice("Test Choice",Choices);
        
        // add the edge row
        RowDialogEdge EdgeRow = Dialog.AddRowEdge("Test Edge",Net.GetEdgeIndexStrings());
        EdgeRow.PutHelpFile("DialogSolverEdge.html"); // add a help button for edges
        
        // add the node row
        RowDialogNode NodeRow = Dialog.AddRowNode("Test Node",Net.GetNodeIndexStrings());

        // add Cancel and OK buttons
        Dialog.AddButton("Cancel");
        Dialog.AddButton("OK");
        
        // interact with the user        
        String pushed_button = Dialog.Interact(); 
        
        // retrieve results
        TestText = TextRow.GetText();
        TestFilename = FilenameRow.GetFilename();
        TestChoice = ChoiceRow.GetChoice();
        TestNode = NodeRow.GetNode();
        TestEdge = EdgeRow.GetEdge();
        
        // dispose of the dialog box
        Dialog.dispose();
        
        // returns true if the OK button was pushed
        return pushed_button.equals("OK");
    }   
  </pre>
 * @see RowDialogChoice
 * @see RowDialogEdge
 * @see RowDialogEntry
 * @see RowDialogFilename
 * @see RowDialogNode
 * @see RowDialogText
 *
 * @version 	4.0a 2004 08 27
 * @author 	    D. S. Dilworth

 */
 

public class RowDialogMain extends Frame implements ActionListener, MouseMotionListener, MouseListener
{
    int NameWidth;
    int InfoWidth;
    int ClickWidth;
    int HelpWidth;
    int Height;
    Button Buttons[];
    int ButtonCount;
    int ButtonMax;
    String ResultString;
    String HelpFile;
    
    RowDialogEntry Rows[];
    int RowCount;
    int RowMax;
    
    Panel ButtonPanel;
    UserBase UserInfo;
    RowDialogNode NodeClickResult;
    String NodeMessage;
    RowDialogEdge EdgeClickResult;
    String EdgeMessage;
    public RowDialogController ControllerButtonResult;
    String ControllerButtonMessage;
    ExecBase Solver;
    
    Panel SolverPanel;
    
    public RowDialogMain(String name, UserBase user, Point top_left, ExecBase solver)
    {
        super(name);
        UserInfo=user;
        Solver=solver;
        RowMax=10;
        Rows = new RowDialogEntry[RowMax];
        RowCount=0;
        
        NameWidth=0;
        InfoWidth=0;
        ClickWidth=0;
        HelpWidth=0;
        Height=0;
        HelpFile=Solver.BaseUserInfo.Services.HelpFile;
        
        ButtonMax=10;
        Buttons = new Button[ButtonMax];
        ButtonCount=0;
        ResultString=null;
        
        SolverPanel = new Panel();
        add(SolverPanel);
        SolverPanel.addMouseMotionListener(this);
        SolverPanel.addMouseListener(this);
        
        ButtonPanel = new Panel();
        ButtonPanel.addMouseListener(this);
        
        setLocation(top_left);
        
       // HelpFile=null;
    }
    /** internal method */
    public RowDialogController AddRowController(String name, String value, SolverController solver)
    {
        RowDialogController text = new RowDialogController(name,value,this,solver);
        AddRow(text);
        return text;
    }   
    
    /** Adds a text row to the dialog box.
    <br>Use "String RowDialogText.GetText()" to retrieve the text
    */
    public RowDialogText AddRowText(String name, String value)
    {
        RowDialogText text = new RowDialogText(name,value,this);
        AddRow(text);
        return text;
    }
    /** Adds a choice row to the dialog box.
    <br>Use "String RowDialogChoice.GetChoice()" to retrieve the text
    */
    public RowDialogChoice AddRowChoice(String name, String choices[], String nominal)
    {
        RowDialogChoice choice = new RowDialogChoice(name,choices,nominal,this);
        AddRow(choice);
        return choice;
    }
    
    /** Adds a choice row to the diaglog box.
    The choices are an array strings that appear in a pulldown box.  The user can select one of the strings.
    <br>Use "String RowDialogChoice.GetChoice()" to retrieve the text
    */    
    public RowDialogChoice AddRowChoice(String name, String choices[])
    {
        return AddRowChoice(name, choices, null);
    }
    /** Adds a node selection row to the diaglog box.
    The row has a pulldown menu with the available Nodes and a button that allows the user to click on the Node.
    <br>Use "Node RowDialogNode.GetNode()" to retrieve the Node.
    */    
    public RowDialogNode AddRowNode(String name, LinkList choices)
    {
        RowDialogNode choice = new RowDialogNode(name,choices,this);
        AddRow(choice);
        return choice;
    }
    /** Adds an edge selection row to the dialog box.
    The row has a pulldown menu with the available Edges and a button that allows the user to click on the Edge.
    <br>Use "Edge RowDialogEdge.GetEdge()" to retrieve the Edgee.
    */    
    public RowDialogEdge AddRowEdge(String name, LinkList choices)
    {
        RowDialogEdge choice = new RowDialogEdge(name,choices,this);
        AddRow(choice);
        return choice;
    }
    /** Adds a filename selection row to the diaglog box.
    The row has a text area to enter a filename and a button that allows the user to browse and click on a file.
    <br>Use "String RowDialogFilename.GetFilename()" to retrieve the text
    */    
    public RowDialogFilename AddRowFilename(String name,String nominal, boolean savemode)
    {
        
        RowDialogFilename filename = new RowDialogFilename(name,nominal,savemode,this);
        AddRow(filename);
        return filename;
    }
    /** Sets the name of the HTML help file.
     * Note this file is assumed to be in the same folder as the solver.
     * If no filename if specified, the HelpFile for the solver is used*/
    public void PutHelpFile(String helpfile)
    {
        HelpFile=helpfile;
    }
    /** internal method */
    protected GUISolver GetGUI()
    {
        return UserInfo.GetGUI();
    }
    /** internal method */
    protected Network GetNet()
    {
        return UserInfo.Services.GetNet();
    }
    /** internal method */ 
    public void AddRow(RowDialogEntry row)
    {
        row.Build();
        Rows[RowCount++]=row;
    }
    /** Adds a dialog control button to the bottom of the dialog box.
    These buttons are typically used to add [Cancel],[Back],and [OK] types of controls.
    */
    public void AddButton(String name)
    {
        Buttons[ButtonCount]= new Button(name);
        Buttons[ButtonCount].addActionListener(this);
        ButtonCount++;
    }
    /** internal method */
    public Point GetBottomLeft()
    {
        Rectangle bounds = getBounds();
        Point point = new Point();
        point.x = bounds.x;
        point.y = bounds.y+bounds.height;
        return point;
    }
    /** Interacts with the user and returns the string associated with the dialog control button that was pushed. */
    public String Interact()
    {
        boolean loop;
        String event;
        boolean delete_dialog=true;
                 
        SolverPanel.setBackground(Color.lightGray);
        SolverPanel.setLayout(new GridLayout(RowCount+1, 1));
        
        AddComponents();
        pack();
        FixSizes();
        SolverPanel.removeAll();
        pack();
        AddComponents();
        show();
        
        loop=true;
        while(loop)
        {
            loop=false;
            UserInfo.Services.DrawNetworks();
            event=UserInfo.Services.EventWait("Simple",0);
            if(event.equals("Node"))
            {
                Node node =UserInfo.UserMouseNodeInput(NodeMessage);
                NodeClickResult.PutNodeClick(node);
                loop=true;
            }
            if(event.equals("Edge"))
            {
                Edge edge =UserInfo.UserMouseEdgeInput(EdgeMessage);
                EdgeClickResult.PutEdgeClick(edge);
                loop=true;
            }
            if(event.equals("RunButton"))       // not needed but shows flow
            {
                delete_dialog=false;
            }
            if(event.equals("ResultsButton"))      // not needed but shows flow
            {
                delete_dialog=false;
            }
            if(GetResultString()==null)
                loop=true;
            if(event.equals("Null EdgeClick"))
                loop=true;
            if(event.equals("Null NodeClick"))
                loop=true;
        }
        if(delete_dialog)
        {
            int i;
            for(i=0;i<RowCount;i++)
                Rows[i].Terminate();
            setVisible(false);
        }
        return GetResultString();
    }   
    /** internal method */
    public String GetResultString()
    {
        return ResultString;
    }
    /** internal method */
    void AddComponents()
    {
        ButtonPanel.removeAll();
        for(int b=0;b<ButtonCount;b++)
            ButtonPanel.add(Buttons[b]);
        
        for(int i=0;i<RowCount;i++)
            SolverPanel.add(Rows[i]);
        SolverPanel.add(ButtonPanel);
        pack();
        
    }
    /** internal method */
    void FixSizes()
    {
        int i;
        for(i=0;i<RowCount;i++)
            GetMaxSizes(Rows[i]);
        for(i=0;i<RowCount;i++)
            Rows[i].PutSize(Height,NameWidth,InfoWidth,ClickWidth,HelpWidth);
    }
    /** internal method */
    void GetMaxSizes(RowDialogEntry row)
    {
        Dimension d;
        
        d=row.Name.getSize();
        if(d.width>NameWidth)
            NameWidth=d.width;
        if(d.height>Height)
            Height=d.height;
        
        d=row.GetInfoSize();
        if(d.width>InfoWidth)
            InfoWidth=d.width;
        if(d.height>Height)
            Height=d.height;
            
        d=row.GetExtraSize();
        if(d.width>ClickWidth)
            ClickWidth=d.width;
        if(d.height>Height)
            Height=d.height;
            
        d=row.Help.getSize();
        if(d.width>HelpWidth)
            HelpWidth=d.width;
        if(d.height>Height)
            Height=d.height;
            
    }
    /** internal method */
    public void actionPerformed(ActionEvent event)
    {
        boolean result;
        Object obj=event.getSource();
        ResultString=((Button)obj).getLabel();
        UserInfo.Services.EventNotify("Dialog");
    }
    /** internal method */
    void UserMouseNodeInput(String message,RowDialogNode who)
    {
        NodeClickResult=who;
        NodeMessage=message;
        UserInfo.Services.EventNotify("Node");
    }
    /** internal method */
    void UserMouseEdgeInput(String message,RowDialogEdge who)
    {
        EdgeClickResult=who;
        EdgeMessage=message;
        UserInfo.Services.EventNotify("Edge");
    }
    /** internal method */
    void UserRunButton(String message,RowDialogController who)
    {
        ControllerButtonResult=who;
        ControllerButtonMessage=message;
        ResultString="RunButton";
        UserInfo.Services.EventNotify(ResultString);
    }
    void UserResultsButton(String message,RowDialogController who)
    {
        ControllerButtonResult=who;
        ControllerButtonMessage=message;
        ResultString="ResultsButton";
        UserInfo.Services.EventNotify(ResultString);
    }
    
    /** internal method */
    public void mouseDragged(MouseEvent event)
    {
        test();
    }
    /** internal method */
    public void mouseMoved(MouseEvent event)
    {
        test();
    }
    /** internal method */
    public void mouseClicked(MouseEvent event)
    {
        test();
    }
    /** internal method */
    public void mouseEntered(MouseEvent event)
    {
        test();
        requestFocus();
    }
    /** internal method */
    public void mouseExited(MouseEvent event)
    {
        test();
    }
    /** internal method */
    public void mousePressed(MouseEvent event)
    {
        test();
    }
    /** internal method */
    public void mouseReleased(MouseEvent event)
    {
        test();
    }
    /** internal method */
    int test()
    {
        int i=0;
        return i;
    }
}

