package giden.contrib.NU_IEMS_452_W04;
import giden.CORE.*;
import giden.GRDS.*;
import java.awt.Color;

import giden.GUI.GidenSolverInput;
import giden.GUI.SolverInputItem;

public class GlobalMinCutSolver extends ExecBase
{
    UserBase UI;             // user interactions
    Network Net;             // access to network
    
    AnimationSet ViewSet;    // ViewSet data member
    AnimationSet OrigSet;    // OrigSet data member
    AnimationSet CurrCutSet; // CurrCutSet data member
    AnimationSet OrderedSet; // OrderedSet data member

    private FinalSet EndSet;

    private GidenSolverInput input;
    private EdgeArray Capacity;
    
    /** Initializes invariant objects */
    public GlobalMinCutSolver (UserBase userinfo)
    {
        super(userinfo); // mandatory call to the base class
        UI =  userinfo;  // local access for user interactions

        EndSet = UI.CreateFinalSet("Minimum Cut",Color.orange);
        
        // create the animation sets
        OrigSet    = UI.CreateAnimationSet("",Color.lightGray);
        OrderedSet = UI.CreateAnimationSet("Ordered Set",Color.yellow);
        ViewSet    = UI.CreateAnimationSet("View Set",Color.green);
        CurrCutSet = UI.CreateAnimationSet("Current Cut",Color.red);
        UI.CreateAnimationButtons();

        input = new GidenSolverInput();
        input.addItem(SolverInputItem.INTEGER_EDGE_ARRAY,"Capacity");
    }
    
    /** Prepares for solver execution */
    public boolean Setup(Network net)
    {
        Net=net;                        // local access to the network
        UI.PutDirected(false);

        if (!UI.NullBase) {
            UI.CreateSolverInputDialog("Global Minimum Cut Solver", this, Net, input);
            if (!input.isValidated()) {
                return false;
            } /* end if */
            /* extract input info */
            Capacity = (EdgeArray) input.getItemData("Capacity");
        } /* end if */
        if (Capacity == null) {
            Capacity = Net.GetEdgeArray("Capacity");
        } /* end if */
        if (Capacity == null) {
            return false;
        } /* end if */
        UI.StatusLine("Initialization"); // status line
        UI.MoveAll(OrigSet);             // nodes/edges -> OrigSet
        UI.ShowPseudoCodeText("start");  // pseudocode window
        return true;
    }

    /** Validates the user input */
    public boolean ValidateInput() {return true;}    

    /** Executes the solver */
    public boolean Execute()
    {
        UI.StatusLine("Executing: Finding first cut"); // status line
        UI.ShowPseudoCodeText("execute");              // pseudocode window
        
        ExecuteGlobalMinCutSolver();                   // execute the solver
        
        UI.ShowPseudoCodeText("end");                  // pseudocode window
        return true;
    }
    
    /** Finds the global minimum cut in the network */
    void ExecuteGlobalMinCutSolver()
    {
        UI.ShowPseudoCodeText("start");
 
        /* Declare variables */
        int num_nodes = Net.GetNodeCount();                      /* number of nodes in the network          */
        NodeArray main = new NodeArray(num_nodes,"main");        /* main array of all nodes                 */
        NodeArray copy = new NodeArray(num_nodes,"copy");        /* array of nodes for legal ordering       */
        NodeArray best_cut = new NodeArray(num_nodes,"bestcut"); /* array of nodes: has label...
                                                                      1  if belongs to minimum cut
                                                                      0  otherwise                          */
        NodeArray seen = new NodeArray(num_nodes,"seen");        /* array of nodes: has label...
                                                                      1  if seen this ordering iteration
                                                                      0  otherwise                          */
        int best_cut_capacity = 0;                               /* capacity of minimum cut                 */
        int num_components = num_nodes;                          /* number of nodes in current iteration    */
        NodeListIter nit = Net.GetNodeListIter();                /* node list iterator for original network */
        EdgeListIter eit = Net.GetEdgeListIter();                /* edge list iterator for original network */
        String current_cut_str = "";                             /* description of current cut capacity     */
        String best_cut_str = "No Cut";                          /* description of minimum cut capacity     */
        int best_cut_updated = 0;                                /* bool value: has value...
                                                                      1  if best cut is ever updated
                                                                      0  otherwise                          */
        int neg_edge_flag = 0;                                   /* bool value: has value...
                                                                      1  if some edge has negative capacity
                                                                      0  otherwise                          */

        /* Initialize network labels and capacity of minimum cut to sum of all edge capacities */
        UI.ShowPseudoCodeText("initialize");
        int index = 1;
        for (Node node=nit.GetFirstNode(); node != null; node=nit.GetNextNode()) {
            main.Put(node,index);
            best_cut.Put(node,0);
            index++;
        } /* end for */
        for (Edge edge=eit.GetFirstEdge(); edge != null; edge=eit.GetNextEdge()) {
            if (Capacity.GetInt(edge) < 0) {
                UI.SingleStep(edge,EndSet);
                neg_edge_flag = 1;
            } /* end if */
            best_cut_capacity += Capacity.GetInt(edge);
        } /* end for */
        if (neg_edge_flag == 1) {
            UI.StatusLine("Usage Error: Capacities must be nonnegative");
            EndSet.PutLabel("Negative Capacity");
            return;
        } /* end if */
        best_cut_capacity += 1;
        
        /* Run main algorithm */
        while (num_components > 1) {
            UI.ShowPseudoCodeText("main loop");  

            /* Create copy of network in its current state */
            for (Node node=nit.GetFirstNode(); node != null; node=nit.GetNextNode()) {
                copy.Put(node,main.GetInt(node));
                UI.PutLabel(node,main.GetInt(node));
            } /* end for */

            UI.Trace(OrigSet);
            if (num_components != num_nodes) {
                UI.ShowPseudoCodeText("contract nodes");
            } /* end if */
            UI.Update();

            /* Get arbitrary first label for legal ordering */
            Node first = nit.GetFirstNode();
            int first_label = copy.GetInt(first);
            for (Node node=nit.GetFirstNode(); node != null; node=nit.GetNextNode()) {
                if (copy.GetInt(node) == first_label) {
                    copy.Put(node,0);
                    UI.PutLabel(node,copy.GetInt(node));
                    UI.SingleStep(node,OrderedSet);
                } /* end if */
            } /* end for */

            UI.ShowPseudoCodeText("initial node");
            UI.Trace(OrderedSet);
            UI.Update();

            /* Initialize variables for legal ordering */
            int capacity = 0;              /* capacity of arcs from current node to previously ordered nodes       */
            int max_capacity = 0;          /* maximum capacity of arcs from a node to previously ordered nodes     */
            int best_label = -1;           /* node corresponding to max_capacity                                   */
            int NextToLast = first_label;  /* next-to-last node in legal ordering: "v_{n-1}"                       */
            int Last       = first_label;  /* last node in legal ordering: "v_n"                                   */
            NodeListIter nit_same = Net.GetNodeListIter(); /* node list iterator for finding nodes with same label */

            /* Complete legal ordering */
            for (int counter = 0; counter < num_components-1; counter++) {
                UI.ShowPseudoCodeText("find legal order");  
             
                /* Initialize variables */
                max_capacity = 0;
                best_label = -1;

                /* Initialize seen array */
                for (Node node=nit.GetFirstNode(); node != null; node=nit.GetNextNode()) {
                    seen.Put(node,0);
                } /* end for */
                seen.Put(first,1);

                /* Scan array of all nodes */
                for (Node node=nit.GetFirstNode(); node != null; node=nit.GetNextNode()) {

                    /* Determine arc capacities from node to previously ordered nodes if and only if 
                       node is not already ordered and hasn't already been seen                      */
                    if (copy.GetInt(node) != 0 && seen.GetInt(node) == 0) {

                        /* Initialize outgoing arc capacity of current node */
                        capacity = 0;
                        int current_label = copy.GetInt(node);

                        /* Ensure that some node will be chosen next in legal ordering */
                        if (best_label == -1) {
                            best_label = current_label;
                        } /* end if */

                        /* Find nodes with current_label */
                        for (Node node_same=nit_same.GetFirstNode(); node_same != null; node_same=nit_same.GetNextNode()) {

                            /* If node has current_label, then consider its outgoing edges */
                            if (copy.GetInt(node_same) == current_label) {

                                /* Mark node as seen */
                                seen.Put(node_same,1);

                                UI.SingleStep(node_same,ViewSet);
                                EdgeListIter eit_adj = node_same.GetAdjListIter();

                                /* Find outgoing edge capacities of node with current_label */
                                for (Edge e=eit_adj.GetFirstEdge(); e != null; e=eit_adj.GetNextEdge()) {
                                    Node target = e.GetTarget();
                                    Node source = e.GetSource();
                                    if (copy.GetInt(target) == 0 || copy.GetInt(source) == 0) {
                                        capacity += Capacity.GetInt(e);
                                        UI.SingleStep(e,ViewSet);
                                    } /* end if */
                                } /* end for */

                            } /* end if */

                        } /* end for */

                        /* If capacity is best seen so far, then update */
                        if (capacity > max_capacity) {
                            max_capacity = capacity;
                            best_label = current_label;
                        } /* end if */

                        UI.Trace(OrigSet);
                        UI.ShowPseudoCodeText("find capacity");
                        UI.Trace(ViewSet);
                        UI.Update();

                        /* Put all edges into OrigSet */
                        for (Edge e=eit.GetFirstEdge(); e != null; e=eit.GetNextEdge()) {
                            UI.SingleStep(e,OrigSet);
                        } /* end for */

                        /* Put all nodes into OrigSet */
                        NodeListIter n = Net.GetNodeListIter();
                        for (Node no=n.GetFirstNode(); no != null; no=n.GetNextNode()) {
                            if (copy.GetInt(no) != 0) {
                                UI.SingleStep(no,OrigSet);
                            } /* end if */
                        } /* end for */

                    } /* end if */

                } /* end for */

                /* Update next-to-last and last labels */
                NextToLast = Last;
                Last = best_label;

                /* If ordering is complete, then preserve distinct label on last "node" */
                if (counter < num_components-2) {

                    /* Update set of nodes that have been ordered */
                    for (Node node=nit.GetFirstNode(); node != null; node=nit.GetNextNode()) {
                        if (copy.GetInt(node) == best_label) {
                            copy.Put(node,0);
                            UI.PutLabel(node,copy.GetInt(node));
                            UI.SingleStep(node,OrderedSet);
                        } /* end if */
                    } /* end for */

                } /* end if */

                UI.ShowPseudoCodeText("add node"); 
                UI.Trace(OrderedSet);
                UI.Update();

            } /* end for */

            /* Initialize capacity of current cut */
            int current_cut_capacity = 0;

            /* Calculate capacity of the current cut */
            for (Node n=nit.GetFirstNode(); n != null; n=nit.GetNextNode()) {

                /* If current node has distinct label */
                if (copy.GetInt(n) != 0) {
                    
                    UI.SingleStep(n,CurrCutSet);
                    EdgeListIter eit_adj = n.GetAdjListIter();
                    
                    /* Scan all outgoing edges of current node */
                    for (Edge e=eit_adj.GetFirstEdge(); e != null; e=eit_adj.GetNextEdge()) {

                        Node target = e.GetTarget();
                        Node source = e.GetSource();

                        /* Increment capacity of current cut */
                        if (copy.GetInt(target) == 0 || copy.GetInt(source) == 0) {
                            current_cut_capacity += Capacity.GetInt(e);
                            UI.SingleStep(e,CurrCutSet);
                        } /* end if */

                    } /* end for */

                } /* end if */

            } /* end for */

            /* Update minimum cut, if necessary */
            if (current_cut_capacity < best_cut_capacity) {
                best_cut_capacity = current_cut_capacity;
                best_cut_updated  = 1;
                for (Node node=nit.GetFirstNode(); node != null; node=nit.GetNextNode()) {
                    best_cut.Put(node,copy.GetInt(node));
                } /* end for */
            } /* end if */

            Integer current_cut_int = new Integer(current_cut_capacity);
            Integer best_cut_int    = new Integer(best_cut_capacity);
            current_cut_str         = current_cut_int.toString();
            best_cut_str            = best_cut_int.toString();

            UI.StatusLine("Current Cut Capacity: " + current_cut_str + "; Current Minimum Cut Capacity: " + best_cut_str);
            UI.ShowPseudoCodeText("current cut");
            UI.Trace(CurrCutSet);
            UI.Update();

            /* Contract next-to-last and last nodes by contracting labels */
            for (Node node=nit.GetFirstNode(); node != null; node=nit.GetNextNode()) {
                if (main.GetInt(node) == Last) {
                    main.Put(node,NextToLast);
                    UI.PutLabel(node,main.GetInt(node));
                } /* end if */
            } /* end for */

            /* Put all nodes in OrigSet */
            for (Node n=nit.GetFirstNode(); n != null; n=nit.GetNextNode()) {
                UI.SingleStep(n,OrigSet);
            } /* end for */

            /* Put all edges in OrigSet */
            for (Edge e=eit.GetFirstEdge(); e != null; e=eit.GetNextEdge()) {
                UI.SingleStep(e,OrigSet);
            } /* end for */
            
            /* Decrement current number of nodes in graph */
            num_components--;

        } /* end while */

        /* Put components of global minimum cut into EndSet */
        if (best_cut_updated == 0 && num_nodes > 0) {
            Node n=nit.GetFirstNode();
            UI.SingleStep(n,EndSet);
            eit = n.GetAdjListIter();
            for (Edge e=eit.GetFirstEdge(); e != null; e=eit.GetNextEdge()) {
                Node target = e.GetTarget();
                Node source = e.GetSource();
                if (best_cut.GetInt(target) == 0 || best_cut.GetInt(source) == 0) {
                    UI.SingleStep(e,EndSet);
                } /* end if */
            } /* end for */
            for (n=nit.GetFirstNode(); n != null; n=nit.GetNextNode()) {
                UI.PutLabel(n,"-");
            } /* end for */
            if (num_nodes == 1) {
                EndSet.PutLabel("No Cut");
            } /* end if */
            else {
                EndSet.PutLabel("Minimum Cut");
            } /* end else */
        } /* end if */
        else {
            for (Node n=nit.GetFirstNode(); n != null; n=nit.GetNextNode()) {
                if (best_cut.GetInt(n) != 0) {
                    UI.SingleStep(n,EndSet);
                    eit = n.GetAdjListIter();
                    for (Edge e=eit.GetFirstEdge(); e != null; e=eit.GetNextEdge()) {
                        Node target = e.GetTarget();
                        Node source = e.GetSource();
                        if (best_cut.GetInt(target) == 0 || best_cut.GetInt(source) == 0) {
                            UI.SingleStep(e,EndSet);
                        } /* end if */
                    } /* end for */
                } /* end if */
                UI.PutLabel(n,"-");
            } /* end for */
            EndSet.PutLabel("Minimum Cut");
        } /* end else */

        UI.StatusLine("Global Minimum Cut Capacity: " + best_cut_str);
        UI.ShowPseudoCodeText("show global");
        UI.Trace(OrigSet);

    } /* end ExecuteGlobalMinCutSolver */

}
